﻿using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.MultiObjectiveTestFunctions;
using System;
using System.Linq;

namespace HeuristicLab.Algorithms.CMAEvolutionStrategy
{
    class CrowdingIndicator : IIndicator
    {


        int IIndicator.leastContributer<R>(R[] front, Func<R, double[]> extractor, MultiObjectiveTestFunctionProblem problem)
        {
            DoubleMatrix bounds = problem.Bounds;
            int mindex = 1;
            double min = Double.MaxValue;
            double[] pointsums = new double[front.Count()];

            for (int dim = 0; dim < problem.Objectives; dim++)
            {
                double[] arr = front.Select(x => extractor.Invoke(x)[dim]).ToArray();
                Array.Sort(arr);
                double fmax = problem.Bounds[dim % bounds.Rows, 1];
                double fmin = bounds[dim % bounds.Rows, 0];
                int pointIdx = 0;
                foreach (R point in front)
                {
                    double d = 0;
                    int pos = Array.BinarySearch(arr, extractor.Invoke(point)[dim]);

                    d = pos != 0 && pos != arr.Count() - 1 ? (arr[pos + 1] - arr[pos - 1]) / (fmax - fmin) : Double.PositiveInfinity;
                    pointsums[pointIdx] += d;


                    pointIdx++;
                }
            }
            //find min
            for (int pointIdx = 0; pointIdx < pointsums.Length; pointIdx++)
            {
                double d = pointsums[pointIdx];
                if (!Double.IsInfinity(d) && min > d)
                {
                    mindex = pointIdx;
                    min = d;
                }
            }

            return mindex;
        }
    }
}
